=== modified file 'debian/changelog'
--- debian/changelog	2009-11-25 11:40:43 +0000
+++ debian/changelog	2009-12-19 18:00:22 +0000
@@ -1,3 +1,13 @@
+hardening-wrapper (1.18) unstable; urgency=low
+
+  * debian/{control,rules}: add "hardening-includes" for use in other
+    Debian rules files.
+  * debian/rules, hardening.make: relocate/enhance architecture logic
+    to common makefile include file.
+  * tests/*: update to test both wrapper and include style.
+
+ -- Kees Cook <kees@debian.org>  Sat, 19 Dec 2009 18:00:22 -0800
+
 hardening-wrapper (1.17) unstable; urgency=low
 
   * Add Conflicts on binutils-gold, which also uses diversions against

=== modified file 'debian/control'
--- debian/control	2009-11-25 11:40:43 +0000
+++ debian/control	2009-12-19 18:00:22 +0000
@@ -4,7 +4,7 @@
 Build-Depends: debhelper (>= 5.0.0)
 Maintainer: Package Hardening <hardening-discuss@lists.alioth.debian.org>
 Uploaders: Kees Cook <kees@debian.org>, Moritz Muehlenhoff <jmm@debian.org>
-Standards-Version: 3.8.2
+Standards-Version: 3.8.3
 Homepage: http://wiki.debian.org/Hardening
 Vcs-Svn: svn://svn.debian.org/hardening/hardening-wrapper
 Vcs-Browser: http://svn.debian.org/wsvn/hardening/hardening-wrapper/
@@ -17,3 +17,10 @@
  Replaces gcc, g++, and ld with wrapper scripts that set security hardening
  compilation flags, as an alternative to changing gcc specs.  This is an
  experimental package intended for build testing.
+
+Package: hardening-includes
+Architecture: all
+Description: Makefile for enabling compiler flags for security hardening
+ Makefile to be included in Debian rules files.  CFLAGS and LDFLAGS
+ can be extended to include the respective HARDENING_* variables which
+ contain architecture-validated security hardening compiler options.

=== added file 'debian/hardening-wrapper.manpages'
--- debian/hardening-wrapper.manpages	1970-01-01 00:00:00 +0000
+++ debian/hardening-wrapper.manpages	2009-12-19 18:00:22 +0000
@@ -0,0 +1,4 @@
+hardened-cc.1
+hardened-c++.1
+hardened-ld.1
+hardening-wrapper.1

=== added file 'debian/hardening-wrapper.postinst'
--- debian/hardening-wrapper.postinst	1970-01-01 00:00:00 +0000
+++ debian/hardening-wrapper.postinst	2009-12-19 18:00:22 +0000
@@ -0,0 +1,23 @@
+#!/bin/sh -e
+
+undiv()
+{
+	dpkg-divert --package hardening-wrapper \
+		--rename --remove /usr/bin/"$1" || true
+}
+
+# Clean up old diversions from version 1.1
+if [ "$1" = "configure" ] && [ -n "$2" ] && dpkg --compare-versions "$2" lt "1.2"; then
+	for i in gcc g++
+	do
+		undiv "$i"
+	done
+
+	eval $(dpkg-architecture -a)
+	for i in gcc g++
+	do
+		undiv "${DEB_BUILD_GNU_TYPE}-$i"
+	done
+fi
+
+#DEBHELPER#

=== added file 'debian/hardening-wrapper.postrm'
--- debian/hardening-wrapper.postrm	1970-01-01 00:00:00 +0000
+++ debian/hardening-wrapper.postrm	2009-12-19 18:00:22 +0000
@@ -0,0 +1,20 @@
+#!/bin/sh -e
+
+undiv()
+{
+	dpkg-divert --package hardening-wrapper \
+		--rename --remove /usr/bin/"$1" || true
+}
+
+if [ "$1" = "remove" ]; then
+	for ver in 4.1 4.2 4.3 4.4
+	do
+		for i in gcc g++
+		do
+			undiv "$i-$ver"
+		done
+	done
+	undiv ld
+fi
+
+#DEBHELPER#

=== added file 'debian/hardening-wrapper.preinst'
--- debian/hardening-wrapper.preinst	1970-01-01 00:00:00 +0000
+++ debian/hardening-wrapper.preinst	2009-12-19 18:00:22 +0000
@@ -0,0 +1,20 @@
+#!/bin/sh -e
+
+dodiv()
+{
+	dpkg-divert --package hardening-wrapper --add --rename \
+		--divert /usr/bin/"$1".real /usr/bin/"$1"
+}
+
+if [ "install" = "$1" ] || [ "upgrade" = "$1" ]; then
+	for ver in 4.1 4.2 4.3 4.4
+	do
+		for i in gcc g++
+		do
+			dodiv "$i-$ver"
+		done
+	done
+	dodiv ld
+fi
+
+#DEBHELPER#

=== removed file 'debian/manpages'
--- debian/manpages	2008-01-22 16:48:49 +0000
+++ debian/manpages	1970-01-01 00:00:00 +0000
@@ -1,4 +0,0 @@
-hardened-cc.1
-hardened-c++.1
-hardened-ld.1
-hardening-wrapper.1

=== removed file 'debian/postinst'
--- debian/postinst	2008-01-23 02:56:57 +0000
+++ debian/postinst	1970-01-01 00:00:00 +0000
@@ -1,23 +0,0 @@
-#!/bin/sh -e
-
-undiv()
-{
-	dpkg-divert --package hardening-wrapper \
-		--rename --remove /usr/bin/"$1" || true
-}
-
-# Clean up old diversions from version 1.1
-if [ "$1" = "configure" ] && [ -n "$2" ] && dpkg --compare-versions "$2" lt "1.2"; then
-	for i in gcc g++
-	do
-		undiv "$i"
-	done
-
-	eval $(dpkg-architecture -a)
-	for i in gcc g++
-	do
-		undiv "${DEB_BUILD_GNU_TYPE}-$i"
-	done
-fi
-
-#DEBHELPER#

=== removed file 'debian/postrm'
--- debian/postrm	2008-11-20 23:25:52 +0000
+++ debian/postrm	1970-01-01 00:00:00 +0000
@@ -1,20 +0,0 @@
-#!/bin/sh -e
-
-undiv()
-{
-	dpkg-divert --package hardening-wrapper \
-		--rename --remove /usr/bin/"$1" || true
-}
-
-if [ "$1" = "remove" ]; then
-	for ver in 4.1 4.2 4.3 4.4
-	do
-		for i in gcc g++
-		do
-			undiv "$i-$ver"
-		done
-	done
-	undiv ld
-fi
-
-#DEBHELPER#

=== removed file 'debian/preinst'
--- debian/preinst	2008-11-20 23:25:52 +0000
+++ debian/preinst	1970-01-01 00:00:00 +0000
@@ -1,20 +0,0 @@
-#!/bin/sh -e
-
-dodiv()
-{
-	dpkg-divert --package hardening-wrapper --add --rename \
-		--divert /usr/bin/"$1".real /usr/bin/"$1"
-}
-
-if [ "install" = "$1" ] || [ "upgrade" = "$1" ]; then
-	for ver in 4.1 4.2 4.3 4.4
-	do
-		for i in gcc g++
-		do
-			dodiv "$i-$ver"
-		done
-	done
-	dodiv ld
-fi
-
-#DEBHELPER#

=== modified file 'debian/rules'
--- debian/rules	2009-10-29 21:21:55 +0000
+++ debian/rules	2009-12-19 18:00:22 +0000
@@ -7,25 +7,9 @@
 DEB_HOST_ARCH_OS := $(shell dpkg-architecture -qDEB_HOST_ARCH_OS 2>/dev/null)
 DEB_HOST_ARCH_CPU := $(shell dpkg-architecture -qDEB_HOST_ARCH_CPU 2>/dev/null)
 
-# Calculate arch-specific defaults
-DEFAULT_PIE=0
-ifneq (,$(findstring :$(DEB_HOST_ARCH_OS):,:linux:knetbsd:))
-  # PIE enabled only on linux/knetbsd
-  ifeq (,$(findstring :$(DEB_HOST_ARCH_CPU):,:hppa:m68k:arm:mips:mipsel:))
-    # disabled on hppa, m68k (where it doesn't work).
-    # disabled on mips/mipsel due to bug 532821 in toolchain
-    # disabled for arm since there is no randomization support in ARM kernels
-    DEFAULT_PIE=1
-  endif
-endif
-DEFAULT_STACKPROT=1
-ifneq (,$(findstring :$(DEB_HOST_ARCH_CPU):,:ia64:alpha:mips:mipsel:hppa:arm:))
-  # Stack protector disabled on ia64, alpha, mips, mipsel, hppa.
-  #   "warning: -fstack-protector not supported for this target"
-  # Stack protector disabled on arm.
-  #   compiler supports it incorrectly (leads to SEGV)
-  DEFAULT_STACKPROT=0
-endif
+include hardening.make
+DEFAULT_PIE:=$(DEB_BUILD_HARDENING_PIE)
+DEFAULT_STACKPROT:=$(DEB_BUILD_HARDENING_STACKPROTECTOR)
 
 build: build-stamp test
 build-stamp:
@@ -72,16 +56,17 @@
 	dh_testroot
 	dh_clean -k
 	dh_installdirs usr/bin
-	dh_installdirs -A usr/share/lintian/overrides
+	dh_installdirs -phardening-wrapper -A usr/share/lintian/overrides
 	install -m644 debian/lintian.overrides debian/hardening-wrapper/usr/share/lintian/overrides/hardening-wrapper
 	install build-tree/hardened-cc build-tree/hardened-c++ build-tree/hardened-ld debian/hardening-wrapper/usr/bin
-	# Build links
-	for ver in 4.1 4.2 4.3 4.4; do dh_link \
+	# programatically Build links
+	for ver in 4.1 4.2 4.3 4.4; do dh_link -phardening-wrapper \
 		usr/bin/hardened-cc usr/bin/gcc-$$ver \
 		usr/bin/hardened-c++ usr/bin/g++-$$ver \
 		;\
 	done
-	dh_link usr/bin/hardened-ld usr/bin/ld
+	dh_link -phardening-wrapper usr/bin/hardened-ld usr/bin/ld
+	install -m644 -D hardening.make debian/hardening-includes/usr/share/hardening-includes/hardening.make
 
 
 # Build architecture-dependent files here.
@@ -109,8 +94,8 @@
 	dh_builddeb
 
 # Build architecture-independent files here.
-binary-indep: binary-arch
-	# Nothing to do here
+binary-indep:
+	# Nothing
 
 source diff:
 	@echo >&2 'source and diff are obsolete - use dpkg-source -b'; false

=== modified file 'hardened-cc.1'
--- hardened-cc.1	2009-07-22 19:52:00 +0000
+++ hardened-cc.1	2009-12-19 18:00:22 +0000
@@ -1,4 +1,4 @@
-.TH BUILDER-CC 1 "2002-03-02" "Debian Project" "Debian GNU/Linux"
+.TH HARDENED-CC 1 "2008-01-08" "Debian Project" "Debian GNU/Linux"
 .SH NAME
 hardened-cc \- gcc wrapper to enforce hardening toolchain improvements
 .SH SYNOPSIS

=== modified file 'hardened-ld.1'
--- hardened-ld.1	2009-07-22 19:52:00 +0000
+++ hardened-ld.1	2009-12-19 18:00:22 +0000
@@ -1,4 +1,4 @@
-.TH BUILDER-CC 1 "2002-03-02" "Debian Project" "Debian GNU/Linux"
+.TH HARDENED-LD 1 "2008-01-08" "Debian Project" "Debian GNU/Linux"
 .SH NAME
 hardened-ld \- linker wrapper to enforce hardening toolchain improvements
 .SH SYNOPSIS

=== added file 'hardening.make'
--- hardening.make	1970-01-01 00:00:00 +0000
+++ hardening.make	2009-12-19 18:00:22 +0000
@@ -0,0 +1,96 @@
+# -*- Makefile -*-, you silly Emacs!
+# vim: set ft=make:
+#
+# This file is intended to be included in a Debian rules file so that the
+# the calculated HARDENING_CFLAGS and HARDENING_LDFLAGS from this makefile
+# can by used in the package's CFLAGS and LDFLAGS to harden the security of
+# a package's resulting binaries.  For example:
+#
+#   include /usr/share/hardening-includes/hardening.make
+#   CFLAGS += $(HARDENING_CFLAGS)
+#   LDFLAGS += $(HARDENING_LDFLAGS)
+#
+# By default, all hardening options that are valid for a given architecture
+# are enabled.  The following can be set before or after including this
+# makefile:
+#   To disable all hardening:        DEB_BUILD_HARDENING:=0
+#   To disable PIE:                  DEB_BUILD_HARDENING_PIE:=0
+#   To disable stack protector:      DEB_BUILD_HARDENING_STACKPROTECTOR:=0
+#   To disable Fortify Source:       DEB_BUILD_HARDENING_FORTIFY:=0
+#   To disable format string checks: DEB_BUILD_HARDENING_FORMAT:=0
+#   To disable readonly relations:   DEB_BUILD_HARDENING_RELRO:=0
+#   To disable BIND_NOW:             DEB_BUILD_HARDENING_BINDNOW:=0
+#
+# For more details, see https://wiki.debian.org/Hardening
+
+DEB_HOST_ARCH_OS ?= $(shell dpkg-architecture -qDEB_HOST_ARCH_OS 2>/dev/null)
+DEB_HOST_ARCH_CPU ?= $(shell dpkg-architecture -qDEB_HOST_ARCH_CPU 2>/dev/null)
+
+DEB_BUILD_HARDENING ?= 1
+
+ifneq (,$(findstring :$(DEB_HOST_ARCH_OS):,:linux:knetbsd:))
+  # PIE enabled only on linux/knetbsd
+  ifeq (,$(findstring :$(DEB_HOST_ARCH_CPU):,:hppa:m68k:arm:mips:mipsel:))
+    # disabled on hppa, m68k (where it doesn't work).
+    # disabled on mips/mipsel due to bug 532821 in toolchain
+    # disabled for arm since there is no randomization support in ARM kernels
+    DEB_BUILD_HARDENING_PIE ?= 1
+  endif
+endif
+DEB_BUILD_HARDENING_PIE ?= 0
+
+ifneq (,$(findstring :$(DEB_HOST_ARCH_CPU):,:ia64:alpha:mips:mipsel:hppa:arm:))
+  # Stack protector disabled on ia64, alpha, mips, mipsel, hppa.
+  #   "warning: -fstack-protector not supported for this target"
+  # Stack protector disabled on arm.
+  #   compiler supports it incorrectly (leads to SEGV)
+  DEB_BUILD_HARDENING_STACKPROTECTOR ?= 0
+endif
+DEB_BUILD_HARDENING_STACKPROTECTOR ?= 1
+
+DEB_BUILD_HARDENING_FORTIFY ?= 1
+DEB_BUILD_HARDENING_FORMAT ?= 1
+DEB_BUILD_HARDENING_RELRO ?= 1
+DEB_BUILD_HARDENING_BINDNOW ?= 1
+
+_HARDENED_PIE_CFLAGS  := -fPIE
+_HARDENED_PIE_LDFLAGS := -fPIE -pie
+
+_HARDENED_STACKPROTECTOR_CFLAGS := -fstack-protector
+
+# Fortify Source requires that -O2 or higher is used, but that should be
+# handled outside of this include file.
+_HARDENED_FORTIFY_CFLAGS  := -D_FORTIFY_SOURCE=2
+
+_HARDENED_FORMAT_CFLAGS   := -Wformat -Wformat-security
+
+_HARDENED_RELRO_LDFLAGS   := -Wl,-z,relro
+
+_HARDENED_BINDNOW_LDFLAGS := -Wl,-z,now
+
+_hardening_enabled = $(if $(filter $(DEB_BUILD_HARDENING), yes 1 on true),\
+$(if $(filter $(1), yes 1 on true),$(2),),)
+
+HARDENING_CFLAGS ?= \
+$(call _hardening_enabled,$(DEB_BUILD_HARDENING_PIE),$(_HARDENED_PIE_CFLAGS)) \
+$(call _hardening_enabled,$(DEB_BUILD_HARDENING_STACKPROTECTOR),$(_HARDENED_STACKPROTECTOR_CFLAGS)) \
+$(call _hardening_enabled,$(DEB_BUILD_HARDENING_FORTIFY),$(_HARDENED_FORTIFY_CFLAGS)) \
+$(call _hardening_enabled,$(DEB_BUILD_HARDENING_FORMAT),$(_HARDENED_FORMAT_CFLAGS)) \
+
+HARDENING_LDFLAGS ?= \
+$(call _hardening_enabled,$(DEB_BUILD_HARDENING_PIE),$(_HARDENED_PIE_LDFLAGS)) \
+$(call _hardening_enabled,$(DEB_BUILD_HARDENING_RELRO),$(_HARDENED_RELRO_LDFLAGS)) \
+$(call _hardening_enabled,$(DEB_BUILD_HARDENING_BINDNOW),$(_HARDENED_BINDNOW_LDFLAGS)) \
+
+# Utility macros designed to allow package maintainer to force a given
+# hardening feature off in certain areas of a build without disabling
+# the option for the entire build.  For example:
+#   CFLAGS += $(HARDENING_CFLAGS)
+#   monkey.o: monkey.c
+#       $(CC) $(CFLAGS) $(HARDENING_DISABLE_STACKPROTECTOR) $< -o $@
+HARDENING_DISABLE_PIE_CFLAGS:=-fno-PIE
+HARDENING_DISABLE_PIE_LDFLAGS:=fno-PIE -nopie
+HARDENING_DISABLE_FORTIFY_CFLAGS:=-U_FORTIFY_SOURCE
+HARDENING_DISABLE_FORMAT_CFLAGS:=-Wno-format-security
+HARDENING_DISABLE_RELRO_LDFLAGS:=-Wl,-z,norelro
+HARDENING_DISABLE_BINDNOW_LDFLAGS:=-Wl,-z,lazy

=== modified file 'tests/Makefile'
--- tests/Makefile	2009-10-29 21:21:55 +0000
+++ tests/Makefile	2009-12-19 18:00:22 +0000
@@ -1,99 +1,5 @@
 #!/usr/bin/make -f
 
-DEB_HOST_ARCH_CPU := $(shell dpkg-architecture -qDEB_HOST_ARCH_CPU 2>/dev/null)
-
-BUILD_TREE=../build-tree
-
-WRAPPED_CC=$(BUILD_TREE)/hardened-cc
-WRAPPED_CXX=$(BUILD_TREE)/hardened-c++
-WRAPPED_LD=$(BUILD_TREE)/hardened-ld
-
-# It seems that GCC_EXEC_PREFIX does not behave the same across all
-# architectures, so we need to pass the -B flag instead.
-CC=$(WRAPPED_CC) -B $(BUILD_TREE)/
-
-WRAPPERS=$(WRAPPED_CC) $(WRAPPED_LD) $(WRAPPED_CXX)
-
-
-SYMLINKED_LD=$(BUILD_TREE)/ld
-
-HELLO=hello.c
-
-TEST_REQS=$(HELLO) $(WRAPPERS) $(SYMLINKED_LD)
-
-export HARDENING_USE_USR_BIN=1
-export DEB_BUILD_HARDENING=1
-export DEB_BUILD_HARDENING_DEBUG=1
-
-TESTS=\
-	syntax.stamp \
-	$(BUILD_TREE)/test-stock \
-	$(BUILD_TREE)/test-compiled \
-	$(SYMLINKED_LD) \
-	$(BUILD_TREE)/test-linked \
-	$(BUILD_TREE)/test-fPIC-direct \
-	$(BUILD_TREE)/test-fPIC
-
-check: $(TESTS)
-
-clean:
-	rm -f $(TESTS)
-
-syntax.stamp: $(WRAPPERS)
-	# Test basic perl syntax
-	for script in $(WRAPPERS); do perl -c $$script; done
-	touch $@
-
-##########
-# Compilation and linking results tests
-
-$(BUILD_TREE)/test-stock: $(HELLO) $(WRAPPERS)
-	# Compiler and linker options disabled.
-	DEB_BUILD_HARDENING=0 $(CC) -o $@ $<
-	readelf -ld $@
-	$@
-
-$(BUILD_TREE)/test-compiled: $(HELLO) $(WRAPPERS)
-	# Compiler options enabled. (linker is system-default)
-	$(CC) -o $@ $<
-	readelf -ld $@
-	# Run twice to show off PIE, if available in kernel
-	$@
-	$@
-
-$(SYMLINKED_LD): $(WRAPPED_LD)
-	# Enable symlink for ld to trick gcc into doing wrapped linking
-	(cd $(BUILD_TREE) && ln -s hardened-ld ld)
-
-$(BUILD_TREE)/test-linked: $(TEST_REQS)
-	# Compiler and linker options enabled.
-	$(CC) -o $@ $<
-	readelf -ld $@
-	# Run twice to show off PIE, if available in kernel
-	$@
-	$@
-ifeq (,$(findstring :$(DEB_HOST_ARCH_CPU):,:ia64:hppa:))
-	# Test for RELRO
-	readelf -l $@ | grep GNU_RELRO
-endif
-	# Test for BIND_NOW
-	readelf -d $@ | grep BIND_NOW
-
-
-##########
-# Compiler arg calling style tests
-
-# cmake likes to pass -fPIC to everything, which broke pre-1.10 wrappers
-$(BUILD_TREE)/test-fPIC-direct: $(TEST_REQS)
-	# Build directly with -fPIC already defined
-	$(CC) -fPIC -o $@ $<
-	$@
-
-$(BUILD_TREE)/test-fPIC.o: $(TEST_REQS)
-	# Build .o with -fPIC already defined
-	$(CC) -fPIC -o $@ -c $<
-
-$(BUILD_TREE)/test-fPIC: $(BUILD_TREE)/test-fPIC.o $(TEST_REQS)
-	# Link .o with -fPIC already defined
-	$(CC) -fPIC -o $@ $<
-	$@
+check:
+	$(MAKE) -f Makefile.wrapper $@
+	$(MAKE) -f Makefile.includes $@

=== added file 'tests/Makefile.common'
--- tests/Makefile.common	1970-01-01 00:00:00 +0000
+++ tests/Makefile.common	2009-12-19 18:00:22 +0000
@@ -0,0 +1,105 @@
+#!/usr/bin/make -f
+
+include ../hardening.make
+
+BUILD_TREE:=../build-tree
+
+SYMLINKED_LD?=
+WRAPPERS?=
+SYNTAX_STAMP?=
+
+CFLAGS += -O2
+LDFLAGS?=
+
+HELLO=hello.c
+
+TEST_REQS=$(HELLO) $(WRAPPERS) $(SYMLINKED_LD)
+
+TESTS=\
+	$(SYNTAX_STAMP) \
+	$(BUILD_TREE)/$(NAME)-test-stock \
+	$(BUILD_TREE)/$(NAME)-test-compiled \
+	$(SYMLINKED_LD) \
+	$(BUILD_TREE)/$(NAME)-test-linked \
+	$(BUILD_TREE)/$(NAME)-test-fPIC-direct \
+	$(BUILD_TREE)/$(NAME)-test-fPIC
+
+check: $(TESTS)
+
+clean:
+	rm -f $(TESTS)
+
+##########
+# Compilation and linking results tests
+
+$(BUILD_TREE)/$(NAME)-test-stock: $(HELLO) $(WRAPPERS)
+	# Compiler and linker options disabled.
+	DEB_BUILD_HARDENING=0 $(CC) -o $@ $<
+	readelf -ld $@
+	$@
+
+$(BUILD_TREE)/$(NAME)-test-compiled: $(HELLO) $(WRAPPERS)
+	# Compiler options enabled. (linker is not wrapper)
+	$(CC) $(CFLAGS) $(LDFLAGS) -o $@ $<
+	readelf -ld $@
+	# Run twice to show off PIE, if available in kernel
+	$@
+	$@
+
+$(BUILD_TREE)/$(NAME)-test-linked: $(TEST_REQS)
+	# Compiler and linker options enabled.
+	$(CC) $(CFLAGS) $(LDFLAGS) -o $@ $<
+	readelf -ld $@
+	# Run twice to show off PIE, if available in kernel
+	$@
+	$@
+ifeq (1,$(DEB_BUILD_HARDENING_PIE))
+	# Test PIE
+	readelf -lW $@ | grep '^Elf file type is DYN'
+else
+	# Skipped PIE test
+endif
+ifeq (1,$(DEB_BUILD_HARDENING_STACKPROTECTOR))
+	# Test Stack Protector
+	nm $@ | grep '__stack_chk_fail@@GLIBC'
+else
+	# Skipped Stack Protector test
+endif
+ifeq (1,$(DEB_BUILD_HARDENING_FORTIFY))
+	# Test Fortify
+	nm $@ | grep '__snprintf_chk@@GLIBC'
+else
+	# Skipped Fortify test
+endif
+ifeq (1,$(DEB_BUILD_HARDENING_FORMAT))
+	# Test Format (no-op currently)
+else
+	# Skipped Format test
+endif
+ifeq (,$(findstring :$(DEB_HOST_ARCH_CPU):,:ia64:hppa:))
+	# Test for RELRO
+	readelf -lW $@ | grep GNU_RELRO
+else
+	# Skipping RELRO test
+endif
+	# Test for BIND_NOW
+	readelf -dW $@ | grep BIND_NOW
+
+
+##########
+# Compiler arg calling style tests
+
+# cmake likes to pass -fPIC to everything, which broke pre-1.10 wrappers
+$(BUILD_TREE)/$(NAME)-test-fPIC-direct: $(TEST_REQS)
+	# Build directly with -fPIC already defined
+	$(CC) -fPIC $(CFLAGS) $(LDFLAGS) -o $@ $<
+	$@
+
+$(BUILD_TREE)/$(NAME)-test-fPIC.o: $(TEST_REQS)
+	# Build .o with -fPIC already defined
+	$(CC) -fPIC $(CFLAGS) $(LDFLAGS) -o $@ -c $<
+
+$(BUILD_TREE)/$(NAME)-test-fPIC: $(BUILD_TREE)/$(NAME)-test-fPIC.o $(TEST_REQS)
+	# Link .o with -fPIC already defined
+	$(CC) -fPIC $(CFLAGS) $(LDFLAGS) -o $@ $<
+	$@

=== added file 'tests/Makefile.includes'
--- tests/Makefile.includes	1970-01-01 00:00:00 +0000
+++ tests/Makefile.includes	2009-12-19 18:00:22 +0000
@@ -0,0 +1,10 @@
+#!/usr/bin/make -f
+
+NAME:=includes
+
+include ../hardening.make
+
+CFLAGS  += $(HARDENING_CFLAGS)
+LDFLAGS += $(HARDENING_LDFLAGS)
+
+include Makefile.common

=== added file 'tests/Makefile.wrapper'
--- tests/Makefile.wrapper	1970-01-01 00:00:00 +0000
+++ tests/Makefile.wrapper	2009-12-19 18:00:22 +0000
@@ -0,0 +1,31 @@
+#!/usr/bin/make -f
+
+WRAPPED_CC=$(BUILD_TREE)/hardened-cc
+WRAPPED_CXX=$(BUILD_TREE)/hardened-c++
+WRAPPED_LD=$(BUILD_TREE)/hardened-ld
+
+# It seems that GCC_EXEC_PREFIX does not behave the same across all
+# architectures, so we need to pass the -B flag instead.
+CC=$(WRAPPED_CC) -B $(BUILD_TREE)/
+
+WRAPPERS=$(WRAPPED_CC) $(WRAPPED_LD) $(WRAPPED_CXX)
+
+SYNTAX_STAMP=syntax.stamp
+SYMLINKED_LD=$(BUILD_TREE)/ld
+
+export HARDENING_USE_USR_BIN=1
+export DEB_BUILD_HARDENING=1
+export DEB_BUILD_HARDENING_DEBUG=1
+
+NAME:=wrapper
+
+include Makefile.common
+
+$(SYNTAX_STAMP): $(WRAPPERS)
+	# Test basic perl syntax
+	for script in $(WRAPPERS); do perl -c $$script; done
+	touch $@
+
+$(SYMLINKED_LD): $(WRAPPED_LD)
+	# Enable symlink for ld to trick gcc into doing wrapped linking
+	(cd $(BUILD_TREE) && ln -s hardened-ld ld)

